extends Node

var game_state
var town_safe = true
var next_golem: Array[String] = ["", "", ""]

@export var glob: PackedScene
@export var golem: PackedScene
var glob_count = 0

@onready var level = $Level
@onready var main_menu = $MainMenu
@onready var intro_1 = $IntroductionPageOne
@onready var intro_2 = $IntroductionPageTwo
@onready var intro_3 = $IntroductionPageThree
@onready var completion = $Completion

@onready var fire_button = $Level/Interface/Elements/FireButton
@onready var water_button = $Level/Interface/Elements/WaterButton
@onready var air_button = $Level/Interface/Elements/AirButton
@onready var earth_button = $Level/Interface/Elements/EarthButton
@onready var build_button = $Level/Interface/BuildButton

var element_colors: Array[Color] = [Color.RED, Color.DODGER_BLUE, Color.SKY_BLUE, Color.SADDLE_BROWN]
@onready var summon_defence = $Level/SummoningStone/Golem/Defence
@onready var summon_feet = $Level/SummoningStone/Golem/Feet
@onready var summon_arms = $Level/SummoningStone/Golem/Arms
@onready var summon_golem = $Level/SummoningStone/Golem
var summon_defence_scale = 1
var summon_arms_scale = 1
var summon_feet_scale = 1

@onready var home_spawn = $Level/Field/HomeSpawn
@onready var enemy_spawn = $Level/Field/EnemySpawn
@onready var blue_wizard = $Level/Field/BlueWizard
var red_wizard

@onready var fire_bar = $Level/Interface/Elements/FireButton/CoolDownBar
@onready var water_bar = $Level/Interface/Elements/WaterButton/CoolDownBar
@onready var air_bar = $Level/Interface/Elements/AirButton/CoolDownBar
@onready var earth_bar = $Level/Interface/Elements/EarthButton/CoolDownBar
@onready var fire_counter = $Level/Interface/Elements/FireButton/CounterBackground/Counter
@onready var water_counter = $Level/Interface/Elements/WaterButton/CounterBackground/Counter
@onready var air_counter = $Level/Interface/Elements/AirButton/CounterBackground/Counter
@onready var earth_counter = $Level/Interface/Elements/EarthButton/CounterBackground/Counter
@export var cd_tint_color: Color
@export var cd_ready_color: Color
var max_power = 3
var fire: int = 0
var water: int = 0
var air: int = 0
var earth: int = 0
var fire_cd: float = 5
var water_cd: float = 5
var air_cd: float = 5
var earth_cd: float = 5
var current_fire_cd: float = 5
var current_water_cd: float = 5
var current_air_cd: float = 5
var current_earth_cd: float = 5

var glob_respawn_rate: float = 8.55
var glob_timer: float = 5
var glob_spawn_multiplier: float = 1

# audio
@onready var golem_death_sounds = $Audio/Golem/DeathSounds
@onready var golem_hurt_sounds = $Audio/Golem/HurtSounds
@onready var golem_attack_sounds = $Audio/Golem/AttackSounds
@onready var glob_death_sounds = $Audio/Glob/DeathSounds
@onready var glob_hurt_sounds = $Audio/Glob/HurtSounds
@onready var glob_attack_sounds = $Audio/Glob/AttackSounds
@onready var button_deny_sound = $Audio/Interface/ButtonDeny
@onready var button_activate_sound = $Audio/Interface/ButtonSelect

# timers
@onready var show_completion_timer = $Timers/ShowCompletionTimer


func _ready():
	game_state = "intro_1"
	level.visible = true
	intro_1.visible = true
	intro_2.visible = false
	intro_3.visible = false
	main_menu.visible = false
	completion.visible = false
	$Completion/RestartButton.visible = false
	$Completion/LoseStuff.visible = false
	$Completion/WinStuff.visible = false
	red_wizard = get_tree().root.find_child("RedWizard", true, false)


func _process(delta):
	# intro one
	if game_state == "intro_1":
		intro_1.visible = true
	
	# intro two
	if game_state == "intro_2":
		intro_2.visible = true
		intro_1.visible = false

	# intro three
	if game_state == "intro_3":
		intro_3.visible = true
		intro_2.visible = false
	
	# main menu
	if game_state == "mainmenu":
		main_menu.visible = true
		intro_3.visible = false
	
	# run the game
	if game_state == "run":
		main_menu.visible = false
		run_level(delta)
		
	# show win screen
	if game_state == "win":
		if show_completion_timer.is_stopped() and not completion.visible:
			show_completion_timer.start()
		if !$Completion/WinStuff.visible:
			$Completion/Timer.start()
		$Completion/LoseStuff.visible = false
		$Completion/WinStuff.visible = true
	
	if game_state == "loss":
		if show_completion_timer.is_stopped() and not completion.visible:
			show_completion_timer.start()
		if !$Completion/LoseStuff.visible:
			$Completion/Timer.start()
		$Completion/LoseStuff.visible = true
		$Completion/WinStuff.visible = false



func run_level(delta):
	# win condition
	if blue_wizard.health <= 0:
		game_state = "win"
	
	if not town_safe:
		game_state = "loss"
	
	# reduce cool downs
	var reduction_multiplier = 1 - (home_spawn.get_child_count() * 0.4)
	var reduction_speed: float = delta * reduction_multiplier
	current_fire_cd = clamp(current_fire_cd - reduction_speed, 0, fire_cd)
	current_air_cd = clamp(current_air_cd - reduction_speed, 0, air_cd)
	current_water_cd = clamp(current_water_cd - reduction_speed, 0, water_cd)
	current_earth_cd = clamp(current_earth_cd - reduction_speed, 0, earth_cd)
	fire_bar.value = (current_fire_cd / fire_cd) * 100
	water_bar.value = (current_water_cd / water_cd) * 100
	air_bar.value = (current_air_cd / air_cd) * 100
	earth_bar.value = (current_earth_cd / earth_cd) * 100
	set_cd_tint_color(fire_bar, fire_counter)
	set_cd_tint_color(water_bar, water_counter)
	set_cd_tint_color(air_bar, air_counter)
	set_cd_tint_color(earth_bar, earth_counter)
	set_counters(fire_counter, fire)
	set_counters(water_counter, water)
	set_counters(air_counter, air)
	set_counters(earth_counter, earth)
	
	# set current cool down
	if current_fire_cd == 0 and fire < max_power:
		fire += 1
		if fire != max_power:
			current_fire_cd = fire_cd + 0.1
	if current_water_cd == 0 and water < max_power:
		water += 1
		if water != max_power:
			current_water_cd = water_cd + 0.1
	if current_air_cd == 0 and air < max_power:
		air += 1
		if air != max_power:
			current_air_cd = air_cd + 0.1
	if current_earth_cd == 0 and earth < max_power:
		earth += 1
		if earth != max_power:
			current_earth_cd = earth_cd + 0.1
	
	# update build-a-golem button
	if next_golem[0] != "" and next_golem[1] != "" and next_golem[2] != "":
		build_button.disabled = false
	else:
		build_button.disabled = true
	
	# update summoning colors
	var defence_color = Color.WHITE
	if next_golem[0] == "fire":
		defence_color = element_colors[0]
	elif next_golem[0] == "water":
		defence_color = element_colors[1]
	elif next_golem[0] == "air":
		defence_color = element_colors[2]
	elif next_golem[0] == "earth":
		defence_color = element_colors[3]
	var attack_one = Color.WHITE
	if next_golem[1] == "fire":
		attack_one = element_colors[0]
	elif next_golem[1] == "water":
		attack_one = element_colors[1]
	elif next_golem[1] == "air":
		attack_one = element_colors[2]
	elif next_golem[1] == "earth":
		attack_one = element_colors[3]
	var attack_two = Color.WHITE
	if next_golem[2] == "fire":
		attack_two = element_colors[0]
	elif next_golem[2] == "water":
		attack_two = element_colors[1]
	elif next_golem[2] == "air":
		attack_two = element_colors[2]
	elif next_golem[2] == "earth":
		attack_two = element_colors[3]
	summon_defence.modulate = defence_color
	summon_arms.modulate = attack_one
	summon_feet.modulate = attack_two
	
	# update summoning scale / defence
	if next_golem[0] != "":
		summon_defence_scale = clamp(summon_defence_scale + delta * 4, 0, 1)
	else:
		summon_defence_scale = 0
	summon_defence.scale = Vector2(1, 1) * summon_defence_scale
	
	# update summoning scale / arms
	if next_golem[1] != "":
		summon_arms_scale = clamp(summon_arms_scale + delta * 4, 0, 1)
	else:
		summon_arms_scale = 0
	summon_arms.scale = Vector2(1, 1) * summon_arms_scale
	
	# update summoning scale / feet
	if next_golem[2] != "":
		summon_feet_scale = clamp(summon_feet_scale + delta * 4, 0, 1)
	else:
		summon_feet_scale = 0
	summon_feet.scale = Vector2(1, 1) * summon_feet_scale
	
	# spawn globs
	glob_timer -= delta * ((1 + home_spawn.get_child_count()) * 0.5)
	if glob_timer <= 0:
		glob_timer = glob_respawn_rate
		var new_glob = glob.instantiate()
		blue_wizard.animation("cast")
		glob_count += 1
		if glob_count == 5:
			glob_count = 0
			new_glob.glob_size = 1.5
		new_glob.name = "Glob"
		enemy_spawn.add_child(new_glob)


func set_counters(counter, element):
	var number = str(element)
	counter.text = "[center]" + number + "[/center]"


func set_cd_tint_color(bar, counter):
	if counter.text == "[center]0[/center]":
		bar.tint_under = cd_tint_color
	else:
		bar.tint_under = cd_ready_color


func get_multiplier(attacking_element, defending_element):
	var multiplier: float = 1
	# fire
	if attacking_element == 'fire':
		if defending_element == 'water':
			multiplier = 0.5
		elif defending_element == 'earth':
			multiplier = 0.5
		elif defending_element == 'air':
			multiplier = 2
	# water
	if attacking_element == 'water':
		if defending_element == 'fire':
			multiplier = 2
		elif defending_element == 'earth':
			multiplier = 1
		elif defending_element == 'air':
			multiplier = 0.5
	# air
	if attacking_element == 'air':
		if defending_element == 'water':
			multiplier = 2
		elif defending_element == 'earth':
			multiplier = 1
		elif defending_element == 'fire':
			multiplier = 0.5
	# earth
	if attacking_element == 'earth':
		if defending_element == 'water':
			multiplier = 1
		elif defending_element == 'fire':
			multiplier = 2
		elif defending_element == 'air':
			multiplier = 1
	return multiplier


func _on_fire_button_pressed():
	if fire > 0:
		button_activate_sound.play()
		var added = add_element_to_summoning_circle("fire")
		if added:
			fire -= 1
			if current_fire_cd == 0:
				current_fire_cd = fire_cd + 0.1
	else:
		button_deny_sound.play()


func _on_water_button_pressed():
	if water > 0:
		button_activate_sound.play()
		var added = add_element_to_summoning_circle("water")
		if added:
			water -= 1
			if current_water_cd == 0:
				current_water_cd = water_cd + 0.1
	else:
		button_deny_sound.play()


func _on_air_button_pressed():
	if air > 0:
		button_activate_sound.play()
		var added = add_element_to_summoning_circle("air")
		if added:
			air -= 1
			if current_air_cd == 0:
				current_air_cd = air_cd + 0.1
	else:
		button_deny_sound.play()


func _on_earth_button_pressed():
	if earth > 0:
		button_activate_sound.play()
		var added = add_element_to_summoning_circle("earth")
		if added:
			earth -= 1
			if current_earth_cd == 0:
				current_earth_cd = earth_cd + 0.1
	else:
		button_deny_sound.play()


func _on_build_button_pressed():
	if next_golem[0] != "" and next_golem[1] != "" and next_golem[2] != "":
		button_activate_sound.play()
		if game_state == "run":
			red_wizard.animation("cast")
			var new_golem = golem.instantiate()
			new_golem.name = "Golem"
			new_golem.elements = next_golem
			home_spawn.add_child(new_golem)
			next_golem = ["", "", ""]
	else:
		button_deny_sound.play()


func add_element_to_summoning_circle(element):
	#next_golem
	if next_golem[0] == "":
		next_golem[0] = element
	elif next_golem[1] == "":
		next_golem[1] = element
	elif next_golem[2] == "":
		next_golem[2] = element
	else:
		button_deny_sound.play()
		return false
	
	return true


func _on_restart_button_pressed():
	button_activate_sound.play()
	get_tree().reload_current_scene()


func _on_continue_button_pressed():
	button_activate_sound.play()
	game_state = "intro_2"


func _on_continue_button_two_pressed():
	button_activate_sound.play()
	game_state = "intro_3"


func _on_play_button_pressed():
	button_activate_sound.play()
	game_state = "run"


func _on_continue_button_three_pressed():
	button_activate_sound.play()
	game_state = "mainmenu"


func play_sound(collection):
	var count = 0
	var random_int = randi_range(0, collection.get_child_count())
	for sound in collection.get_children():
		if count == random_int:
			sound.play()
		count += 1


func _on_timer_timeout():
	$Completion/RestartButton.visible = true


func _on_show_completion_timer_timeout():
	completion.visible = true
