extends Node2D

@onready var animation_player = $AnimationPlayer
var target
@export var goal_position: float
var move_speed: float = 120
var attack_damage: float = 0.8
var health: int = 10
var element: String
var game
var golems
var attack_range = 80
var damage_applied = false
var goal_distance = 100
var element_colors: Array[Color]
var glob_size = 1
@onready var glob_image = $Glob


func _ready():
	game = get_tree().root.find_child("Game", true, false)
	element_colors = game.element_colors
	golems = get_tree().root.find_child("HomeSpawn", true, false)
	generate_random_element()
	attack_damage = attack_damage * glob_size
	scale = Vector2(glob_size, glob_size)
	health = health * glob_size
	

func _process(delta):
	if health <= 0:
		death()
		return
	
	if target != null:
		# attack target
		attack_target()
	else:
		# try to find a target
		get_target()
		
		# move toward goal
		if global_position.x > goal_distance:
			animation("move")
			if animation_player.current_animation == "move":
				position.x -= delta * move_speed
		else:
			game.town_safe = false


func attack_target():
	animation("attack")
	if damage_applied == false and target != null:
		target.health -= game.get_multiplier(element, target.golem_core) * attack_damage
		damage_applied = true
		game.play_sound(game.glob_attack_sounds)
		if target.health <= 0:
			game.play_sound(game.golem_death_sounds)
			target = null
		else:
			game.play_sound(game.golem_hurt_sounds)


func death():
	animation("death")
	if animation_player.current_animation_position >= 0.15:
		queue_free()


func get_target():
	var golem_objects = golems.get_children()
	for golem in golem_objects:
		if target:
			return
		
		# check if alive
		if golem.health > 0:
			# check if within range
			var target_range = global_position.x - golem.global_position.x
			if target_range < attack_range and target_range > 0:
				target = golem


func animation(animation_name):
	if animation_player.current_animation == animation_name and animation_player.is_playing():
		return
	
	if animation_name == "death":
		animation_player.play(animation_name)
	elif animation_player.current_animation != "attack":
		animation_player.play(animation_name)
		damage_applied = false


func generate_random_element():
	var random_el = randi_range(1, 4)
	
	if random_el == 1:
		element = "fire"
		glob_image.modulate = element_colors[0]
	elif random_el == 2:
		element = "water"
		glob_image.modulate = element_colors[1]
	elif random_el == 3:
		element = "air"
		glob_image.modulate = element_colors[2]
	elif random_el == 4:
		element = "earth"
		glob_image.modulate = element_colors[3]
