extends Node2D

var health = 10
var globs
var target
var attack_range = 100
var move_speed: float = 150
var attack_damage: float = 2
var damage_applied = false
var goal_distance = 920
var blue_wizard
var golem_core: String
var golem_arms: String
var golem_legs: String
var game
@onready var animation_player = $AnimationPlayer
var elements: Array[String]
var element_colors: Array[Color]


func _ready():
	game = get_tree().root.find_child("Game", true, false)
	element_colors = game.element_colors
	globs = get_tree().root.find_child("EnemySpawn", true, false)
	blue_wizard = get_tree().root.find_child("BlueWizard", true, false)
	golem_core = elements[0]
	golem_arms = elements[1]
	golem_legs = elements[2]
	set_element_for_component($Body/Head, elements[0])
	set_element_for_component($Body/Body, elements[0])
	set_element_for_component($"Body/Left Arm", elements[1])
	set_element_for_component($"Body/Right Arm", elements[1])
	set_element_for_component($"Left Foot", elements[2])
	set_element_for_component($"Right Foot", elements[2])


func set_element_for_component(component, element):
	if element == "fire":
		component.modulate = element_colors[0]
	elif element == "water":
		component.modulate = element_colors[1]
	elif element == "air":
		component.modulate = element_colors[2]
	elif element == "earth":
		component.modulate = element_colors[3]


func _process(delta):
	if health <= 0:
		death()
		return
	
	if target != null:
		attack_target()
	else:
		# try to find a target
		get_target()
		
		# move toward goal
		if global_position.x < goal_distance:
			animation("walk", 3.0)
			if animation_player.current_animation == "walk":
				position.x += delta * move_speed
		else:
			if blue_wizard.health > 0:
				attack_the_wizard()
			else:
				animation("cheer")


func death():
	animation("death")
	if animation_player.current_animation_position >= 1.95:
		queue_free()


func attack_target():
	var random_int = randi_range(0, 1)
	var attacking_element = golem_arms
	if random_int == 1:
		animation("attack_punch")
	else:
		animation("attack_kick")
		attacking_element = golem_legs
	if damage_applied == false and target != null:
		game.play_sound(game.golem_attack_sounds)
		var damage_to_apply = game.get_multiplier(attacking_element, golem_core) * attack_damage
		target.health -= damage_to_apply
		damage_applied = true
		if target.health <= 0:
			game.play_sound(game.glob_death_sounds)
			target = null
		else:
			game.play_sound(game.glob_hurt_sounds)


func attack_the_wizard():
	var random_int = randi_range(0, 1)
	if random_int == 1:
		animation("attack_punch")
	else:
		animation("attack_kick")
	if damage_applied == false and blue_wizard.health > 0:
		blue_wizard.health -= 1
		damage_applied = true


func get_target():
	var glob_objects = globs.get_children()
	for glob in glob_objects:
		if target != null:
			return
		
		# check if within range
		var target_range = glob.global_position.x - global_position.x
		if target_range < attack_range and target_range > 0:
			target = glob
			


func animation(animation_name, playback_speed = 1):
	if animation_player.current_animation == animation_name and animation_player.is_playing():
		return
	
	if animation_name == "death":
		animation_player.play(animation_name, -1, playback_speed)
	elif animation_player.current_animation != "attack_punch" and animation_player.current_animation != "attack_kick":
		animation_player.play(animation_name, -1, playback_speed)
		damage_applied = false
